@@ -44,6 +44,7 @@ import logging
 from datetime import datetime, time, date
 from warnings import warn
 
+from six.moves import range
 from logilab.common.modutils import load_module_from_name
 from logilab.common.date import todate, todatetime, utcdatetime, utctime
 from logilab.common.deprecation import deprecated
@@ -75,7 +76,7 @@ def get_dbapi_compliant_module(driver, prefered_drivers=None, quiet=False,
     _ensure_module_loaded(driver)
     try:
         mod = _ADAPTER_DIRECTORY.adapt(driver, prefered_drivers, pywrap=pywrap)
-    except NoAdapterFound, err:
+    except NoAdapterFound as err:
         if not quiet:
             msg = 'No Adapter found for %s, returning native module'
             _LOGGER.warning(msg,  err.objname)
@@ -84,13 +85,15 @@ def get_dbapi_compliant_module(driver, prefered_drivers=None, quiet=False,
 
 def get_connection(driver='postgres', host='', database='', user='',
                   password='', port='', quiet=False, drivers=_PREFERED_DRIVERS,
-                  pywrap=False, extra_args=None):
-    """return a db connection according to given arguments"""
+                  pywrap=False, schema=None, extra_args=None):
+    """return a db connection according to given arguments
+
+    extra_args is an optional string that is appended to the DSN"""
     _ensure_module_loaded(driver)
     module, modname = _import_driver_module(driver, drivers)
     try:
         adapter = _ADAPTER_DIRECTORY.get_adapter(driver, modname)
-    except NoAdapterFound, err:
+    except NoAdapterFound as err:
         if not quiet:
             msg = 'No Adapter found for %s, using default one'
             _LOGGER.warning(msg, err.objname)
@@ -105,7 +108,8 @@ def get_connection(driver='postgres', host='', database='', user='',
     if port:
         port = int(port)
     return adapted_module.connect(host, database, user, password,
-                                  port=port, extra_args=extra_args)
+                                  port=port, schema=schema,
+                                  extra_args=extra_args)
 
 def set_prefered_driver(driver, module, _drivers=_PREFERED_DRIVERS):
     """sets the preferred driver module for driver
@@ -336,7 +340,7 @@ class DBAPIAdapter(object):
                                     self, typecode)
 
     def connect(self, host='', database='', user='', password='', port='',
-                extra_args=None):
+                schema=None, extra_args=None):
         """Wraps the native module connect method"""
         kwargs = {'host' : host, 'port' : port, 'database' : database,
                   'user' : user, 'password' : password}
@@ -752,7 +756,7 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
 
     # allow call to [backup|restore]_commands without previous call to
     # record_connection_information but by specifying argument explicitly
-    dbname = dbhost = dbport = dbuser = dbpassword = dbextraargs = dbencoding = None
+    dbname = dbhost = dbport = dbuser = dbpassword = dbextraargs = dbencoding = dbschema = None
 
     def __init__(self, encoding='utf-8', _cnx=None):
         self.dbencoding = encoding
@@ -768,7 +772,7 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
 
     def record_connection_info(self, dbname, dbhost=None, dbport=None,
                                dbuser=None, dbpassword=None, dbextraargs=None,
-                               dbencoding=None):
+                               dbencoding=None, dbschema=None):
         self.dbname = dbname
         self.dbhost = dbhost
         self.dbport = dbport
@@ -777,6 +781,7 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
         self.dbextraargs = dbextraargs
         if dbencoding:
             self.dbencoding = dbencoding
+        self.dbschema = dbschema
 
     def get_connection(self, initcnx=True):
         """open and return a connection to the database
@@ -793,6 +798,7 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
         cnx = self.dbapi_module.connect(self.dbhost, self.dbname,
                                         self.dbuser,self.dbpasswd,
                                         port=self.dbport,
+                                        schema=self.dbschema,
                                         extra_args=self.dbextraargs)
         if initcnx:
             for hook in SQL_CONNECT_HOOKS.get(self.backend_name, ()):
@@ -818,7 +824,7 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
         raise NotImplementedError('not supported by this DBMS')
 
     def backup_commands(self, backupfile, keepownership=True,
-                        dbname=None, dbhost=None, dbport=None, dbuser=None):
+                        dbname=None, dbhost=None, dbport=None, dbuser=None, dbschema=None):
         """Return a list of commands to backup the given database.
 
         Each command may be given as a list or as a string. In the latter case,
@@ -970,7 +976,7 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
         """
         sql += '\nORDER BY %s' % ','.join(sortterms)
         if sortterms and needwrap:
-            selection = ['T1.C%s' % i for i in xrange(len(selection))]
+            selection = ['T1.C%s' % i for i in range(len(selection))]
             sql = 'SELECT %s FROM (%s) AS T1' % (','.join(selection), sql)
         return sql
 
@@ -1055,6 +1061,14 @@ class _GenericAdvFuncHelper(FTIndexerMixIn):
         """create a new database"""
         raise NotImplementedError('not supported by this DBMS')
 
+    def create_schema(self, cursor, schema, granted_user=None):
+        """create a new database schema"""
+        raise NotImplementedError('not supported by this DBMS')
+
+    def drop_schema(self, cursor, schema):
+        """drop a database schema"""
+        raise NotImplementedError('not supported by this DBMS')
+
     def list_databases(self):
         """return the list of existing databases"""
         raise NotImplementedError('not supported by this DBMS')
