@@ -18,6 +18,8 @@
 # with logilab-database. If not, see <http://www.gnu.org/licenses/>.
 import re
 
+from six import text_type
+
 from logilab.common.textutils import unormalize
 
 REM_PUNC = re.compile(r"[,.;:!?\n\r\t\)\(«»\<\>/\\\|\[\]{}^#@$£_=+\-*&§]")
@@ -39,12 +41,13 @@ def normalize(word):
     stop words are :
        _ single letter
     """
-    assert isinstance(word, unicode), '%r should be unicode' % word
+    assert isinstance(word, text_type), '%r should be unicode' % word
     # do not index single letters
     if len(word) == 1:
         raise StopWord()
     word = unormalize(word.lower(), substitute='')
-    return word.encode('ascii', 'ignore')
+    # we need an ascii-only unicode string, not bytes
+    return word.encode('ascii', 'ignore').decode('ascii')
 
 def normalize_words(rawwords):
     words = []
@@ -152,7 +155,7 @@ CREATE INDEX appears_word_id ON appears (word_id);
     def cursor_index_object(self, uid, obj, cursor):
         position = 0
         # sort for test predictability
-        for weight, words in sorted(obj.get_words().iteritems()):
+        for weight, words in sorted(obj.get_words().items()):
             for word in words:
                 self._save_word(uid, word, position, cursor)
                 position += 1
