@@ -18,10 +18,8 @@
 """Postgres RDBMS support
 
 Supported drivers, in order of preference:
-- psycopg2 (recommended, others are not well tested)
-- psycopg
-- pgdb
-- pyPgSQL
+- psycopg2
+- psycopg2ct
 
 Full-text search based on the tsearch2 extension from the openfts project
 (see http://openfts.sourceforge.net/)
@@ -35,6 +33,8 @@ __docformat__ = "restructuredtext en"
 from os.path import join, dirname, isfile
 from warnings import warn
 
+from six import binary_type
+
 from logilab.common.deprecation import deprecated
 
 from logilab import database as db
@@ -50,45 +50,11 @@ TSEARCH_SCHEMA_PATH = ('/usr/share/postgresql/?.?/contrib/tsearch2.sql', # curre
                        'tsearch2.sql')
 
 
-class _PgdbAdapter(db.DBAPIAdapter):
-    """Simple PGDB Adapter to DBAPI (pgdb modules lacks Binary() and NUMBER)
-    """
-    def __init__(self, native_module, pywrap=False):
-        db.DBAPIAdapter.__init__(self, native_module, pywrap)
-        self.NUMBER = native_module.pgdbType('int2', 'int4', 'serial',
-                                             'int8', 'float4', 'float8',
-                                             'numeric', 'bool', 'money')
-
-    def connect(self, host='', database='', user='', password='', port='', extra_args=None):
-        """Wraps the native module connect method"""
-        if port:
-            warn("pgdb doesn't support 'port' parameter in connect()", UserWarning)
-        kwargs = {'host' : host, 'database' : database,
-                  'user' : user, 'password' : password}
-        cnx = self._native_module.connect(**kwargs)
-        return self._wrap_if_needed(cnx)
-
-
-class _PsycopgAdapter(db.DBAPIAdapter):
-    """Simple Psycopg Adapter to DBAPI (cnx_string differs from classical ones)
-    """
-    def connect(self, host='', database='', user='', password='', port='', extra_args=None):
-        """Handles psycopg connection format"""
-        if host:
-            cnx_string = 'host=%s  dbname=%s  user=%s' % (host, database, user)
-        else:
-            cnx_string = 'dbname=%s  user=%s' % (database, user)
-        if port:
-            cnx_string += ' port=%s' % port
-        if password:
-            cnx_string = '%s password=%s' % (cnx_string, password)
-        cnx = self._native_module.connect(cnx_string)
-        cnx.set_isolation_level(1)
-        return self._wrap_if_needed(cnx)
-
-
-class _Psycopg2Adapter(_PsycopgAdapter):
+class _Psycopg2Adapter(db.DBAPIAdapter):
     """Simple Psycopg2 Adapter to DBAPI (cnx_string differs from classical ones)
+
+    It provides basic support for postgresql schemas :
+    cf. http://www.postgresql.org/docs/current/static/ddl-schemas.html
     """
     # not defined in psycopg2.extensions
     # "select typname from pg_type where oid=705";
@@ -110,6 +76,49 @@ class _Psycopg2Adapter(_PsycopgAdapter):
         db.DBAPIAdapter.__init__(self, native_module, pywrap)
         self._init_psycopg2()
 
+    def connect(self, host='', database='', user='', password='', port='', schema=None, extra_args=None):
+        """Handles psycopg connection format"""
+        args = {}
+        if host:
+            args.setdefault('host', host)
+        if database:
+            args.setdefault('dbname', database)
+        if user:
+            args.setdefault('user', user)
+        if port:
+            args.setdefault('port', port)
+        if password:
+            args.setdefault('password', password)
+        cnx_string = ' '.join('%s=%s' % item for item in args.iteritems())
+        if extra_args is not None:
+            cnx_string += ' ' + extra_args
+        cnx = self._native_module.connect(cnx_string)
+        cnx.set_isolation_level(1)
+        self.set_search_path(cnx, schema)
+        return self._wrap_if_needed(cnx)
+
+    def _schema_exists(self, cursor, schema):
+        cursor.execute('SELECT nspname FROM pg_namespace WHERE nspname=%(s)s',
+                       {'s': schema})
+        return cursor.fetchone() is not None
+
+    def set_search_path(self, cnx, schema):
+        if schema:
+            cursor = cnx.cursor()
+            if not self._schema_exists(cursor, schema):
+                warn("%s schema doesn't exist, search path can't be set" % schema,
+                     UserWarning)
+                return
+            cursor.execute('SHOW search_path')
+            schemas = cursor.fetchone()[0].split(',')
+            if schema not in schemas:
+                schemas.insert(0, schema)
+            else:
+                schemas.pop(schemas.index(schema))
+                schemas.insert(0, schema)
+            cursor.execute('SET search_path TO %s;' % ','.join(schemas))
+            cursor.close()
+
     def _init_psycopg2(self):
         """initialize psycopg2 to use mx.DateTime for date and timestamps
         instead for datetime.datetime"""
@@ -147,37 +156,26 @@ class _Psycopg2CtypesAdapter(_Psycopg2Adapter):
         compat.register()
         _Psycopg2Adapter.__init__(self, native_module, pywrap)
 
-class _PgsqlAdapter(db.DBAPIAdapter):
-    """Simple pyPgSQL Adapter to DBAPI
+class _Psycopg2CffiAdapter(_Psycopg2Adapter):
+    """psycopg2cffi adapter
+
+    cf. https://pypi.python.org/pypi/psycopg2cffi
     """
-    def connect(self, host='', database='', user='', password='', port='', extra_args=None):
-        """Handles psycopg connection format"""
-        kwargs = {'host' : host, 'port': port or None,
-                  'database' : database,
-                  'user' : user, 'password' : password or None}
-        cnx = self._native_module.connect(**kwargs)
-        return self._wrap_if_needed(cnx)
-
-
-    def Binary(self, string):
-        """Emulates the Binary (cf. DB-API) function"""
-        return str
-
-    def __getattr__(self, attrname):
-        # __import__('pyPgSQL.PgSQL', ...) imports the toplevel package
-        return getattr(self._native_module, attrname)
+    def __init__(self, native_module, pywrap=False):
+        # install psycopg2 compatibility
+        from psycopg2cffi import compat
+        compat.register()
+        _Psycopg2Adapter.__init__(self, native_module, pywrap)
 
 
 db._PREFERED_DRIVERS['postgres'] = [
     #'logilab.database._pyodbcwrap',
-    'psycopg2', 'psycopg2ct', 'psycopg', 'pgdb', 'pyPgSQL.PgSQL',
+    'psycopg2', 'psycopg2ct', 'psycopg2cffi',
     ]
 db._ADAPTER_DIRECTORY['postgres'] = {
-    'pgdb' : _PgdbAdapter,
-    'psycopg' : _PsycopgAdapter,
     'psycopg2' : _Psycopg2Adapter,
     'psycopg2ct' : _Psycopg2CtypesAdapter,
-    'pyPgSQL.PgSQL' : _PgsqlAdapter,
+    'psycopg2cffi' : _Psycopg2CffiAdapter,
     }
 
 
@@ -192,7 +190,7 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
         'TZDatetime':'timestamp with time zone'})
     TYPE_CONVERTERS = db._GenericAdvFuncHelper.TYPE_CONVERTERS.copy()
 
-    def pgdbcmd(self, cmd, dbhost, dbport, dbuser, *args):
+    def pgdbcmd(self, cmd, dbhost, dbport, dbuser, dbschema, *args):
         cmd = [cmd]
         cmd += args
         if dbhost or self.dbhost:
@@ -201,6 +199,8 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
             cmd.append('--port=%s' % (dbport or self.dbport))
         if dbuser or self.dbuser:
             cmd.append('--username=%s' % (dbuser or self.dbuser))
+        if dbschema or self.dbschema:
+            cmd.append('--schema=%s' % (dbschema or self.dbschema))
         return cmd
 
     def system_database(self):
@@ -208,8 +208,8 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
         return 'template1'
 
     def backup_commands(self, backupfile, keepownership=True,
-                        dbname=None, dbhost=None, dbport=None, dbuser=None):
-        cmd = self.pgdbcmd('pg_dump', dbhost, dbport, dbuser, '-Fc')
+                        dbname=None, dbhost=None, dbport=None, dbuser=None, dbschema=None):
+        cmd = self.pgdbcmd('pg_dump', dbhost, dbport, dbuser, dbschema, '-Fc')
         if not keepownership:
             cmd.append('--no-owner')
         cmd.append('--file')
@@ -219,7 +219,8 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
 
     def restore_commands(self, backupfile, keepownership=True, drop=True,
                          dbname=None, dbhost=None, dbport=None, dbuser=None,
-                         dbencoding=None):
+                         dbencoding=None, dbschema=None):
+        # XXX what about dbschema ?
         dbname = dbname or self.dbname
         cmds = []
         if drop:
@@ -289,6 +290,17 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
             sql += " ENCODING='%(dbencoding)s'"
         cursor.execute(sql % locals())
 
+    def create_schema(self, cursor, schema, granted_user=None):
+        """create a new database schema"""
+        sql = 'CREATE SCHEMA %s' % schema
+        if granted_user is not None:
+            sql += ' AUTHORIZATION %s' % granted_user
+        cursor.execute(sql)
+
+    def drop_schema(self, cursor, schema):
+        """drop a database schema"""
+        cursor.execute('DROP SCHEMA %s CASCADE' % schema)
+
     def create_language(self, cursor, extlang):
         """postgres specific method to install a procedural language on a database"""
         # make sure plpythonu is not directly in template1
@@ -309,17 +321,26 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
         cursor.execute('SELECT datname FROM pg_database')
         return [r[0] for r in cursor.fetchall()]
 
-    def list_tables(self, cursor):
+    def list_tables(self, cursor, schema=None):
         """return the list of tables of a database"""
-        cursor.execute("SELECT tablename FROM pg_tables")
+        schema = schema or self.dbschema
+        sql = "SELECT tablename FROM pg_tables"
+        if schema:
+            sql += " WHERE schemaname=%(s)s"
+        cursor.execute(sql, {'s': schema})
         return [r[0] for r in cursor.fetchall()]
 
     def list_indices(self, cursor, table=None):
         """return the list of indices of a database, only for the given table if specified"""
         sql = "SELECT indexname FROM pg_indexes"
+        restrictions = []
         if table:
-            sql += " WHERE LOWER(tablename)='%s'" % table.lower()
-        cursor.execute(sql)
+            restrictions.append('LOWER(tablename)=%(table)s')
+        if self.dbschema:
+            restrictions.append('schemaname=%(s)s')
+        if restrictions:
+            sql += ' WHERE %s' % ' AND '.join(restrictions)
+        cursor.execute(sql, {'s': self.dbschema, 'table': table})
         return [r[0] for r in cursor.fetchall()]
 
     # full-text search customization ###########################################
@@ -340,7 +361,7 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
         ctx = {'config': self.config, 'uid': int(uid)}
         tsvectors, size, oversized = [], 0, False
         # sort for test predictability
-        for (weight, words) in sorted(obj.get_words().iteritems()):
+        for (weight, words) in sorted(obj.get_words().items()):
             words = normalize_words(words)
             for i, word in enumerate(words):
                 size += len(word) + 1
@@ -361,8 +382,8 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
                            % ('||'.join(tsvectors), obj.entity_weight), ctx)
 
     def _fti_query_to_tsquery_words(self, querystr):
-        if isinstance(querystr, str):
-            querystr = unicode(querystr, self.dbencoding)
+        if isinstance(querystr, binary_type):
+            querystr = querystr.decode(self.dbencoding)
         words = normalize_words(tokenize_query(querystr))
         # XXX replace '%' since it makes tsearch fail, dunno why yet, should
         # be properly fixed
@@ -419,7 +440,7 @@ class _PGAdvFuncHelper(db._GenericAdvFuncHelper):
         For postgres, install tsearch2 if not installed by the template.
         """
         tstables = []
-        for table in self.list_tables(cursor):
+        for table in self.list_tables(cursor, schema='pg_catalog'):
             if table.startswith('pg_ts'):
                 tstables.append(table)
         if tstables:
