@@ -25,9 +25,10 @@ Supported drivers, in order of preference:
 import os
 import sys
 import shutil
-import StringIO
 from warnings import warn
 
+from six.moves import range
+
 from logilab import database as db
 from logilab.database.sqlserver import _PyodbcAdapter, _AdodbapiAdapter
 
@@ -81,14 +82,14 @@ class _SqlServer2005FuncHelper(db._GenericAdvFuncHelper):
         return [r[0] for r in cursor.fetchall()]
 
     def backup_commands(self, backupfile, keepownership=True,
-                        dbname=None, dbhost=None, dbport=None, dbuser=None):
+                        dbname=None, dbhost=None, dbport=None, dbuser=None, dbschema=None):
         return [[sys.executable, os.path.normpath(__file__),
                  "_SqlServer2005FuncHelper._do_backup", dbhost or self.dbhost,
                  dbname or self.dbname, backupfile]]
 
     def restore_commands(self, backupfile, keepownership=True, drop=True,
                          dbname=None, dbhost=None, dbport=None, dbuser=None,
-                         dbencoding=None):
+                         dbencoding=None, dbschema=None):
         return [[sys.executable, os.path.normpath(__file__),
                 "_SqlServer2005FuncHelper._do_restore", dbhost or self.dbhost,
                  dbname or self.dbname, backupfile],
@@ -219,7 +220,7 @@ AND j.column_id = k.column_id;"""
         if has_limit_or_offset:
             return sql
         if sortterms and needwrap:
-            selection = ['T1.C%s' % i for i in xrange(len(selection))]
+            selection = ['T1.C%s' % i for i in range(len(selection))]
             renamed_sortterms = []
             for term in sortterms:
                 split = term.split()
@@ -326,7 +327,7 @@ AND j.column_id = k.column_id;"""
             time.sleep(1)
             try:
                 size = os.path.getsize(file_share_filename)
-            except OSError, exc:
+            except OSError as exc:
                 self.logger.exception('error accessing %s', file_share_filename)
                 err_count += 1
             if size > prev_size:
